# coding: utf-8
from datetime import datetime
from datetime import time
from itertools import groupby

from edu_to_analytics.service.types import Item
from edu_to_analytics.service.types import Root
from edu_to_analytics.service.utils import PERIOD_DATE
from edu_to_analytics.service.utils import PERIOD_PERIOD
from lxml.etree import tostring
from spyne.util.xml import get_object_as_xml

from extedu.auth.esia_saml.events import AUTH_LOGIN_ESIA
from extedu.core.dict.models import UnitType
from extedu.core.utils.db import get_model_instance
from extedu.employee.models import Employee
from extedu.event_log.events import AUTH_LOGIN
from extedu.event_log.events import MODULE_AUTH
from extedu.event_log.models import EventLog
from extedu.unit.models import Unit
from extedu.webservice.exceptions import SpyneException


def _get_unit_administration(unit):
    if unit.is_administration:
        return unit
    else:
        query = unit.get_ancestors().filter(
            type=UnitType.ADMINISTRATION
        ).order_by('-level')

        return query[0] if query.exists() else None


def _build_record(user_id, day, records):
    first_record = records[0]

    result = Item()

    result.count = len(records)
    result.teacher_id = user_id
    result.fullname = first_record.user_credentials
    result.date = day.strftime('%d.%m.%Y')
    result.times = u', '.join(r.datetime.strftime('%H:%M') for r in records)

    unit = get_model_instance(Unit, first_record.unit_id)
    if unit:
        result.school_id = first_record.unit_id
        result.school_shortname = unit.short_name
        result.school_name = unit.full_name

        administration = _get_unit_administration(unit)
        if administration:
            result.control_unit = administration.short_name
            result.control_unit_id = administration.id
        else:
            result.control_unit = u''
            result.control_unit_id = 0
    else:
        result.control_unit = u''
        result.control_unit_id = 0
        result.school_id = 0
        result.school_shortname = first_record.unit_shortname
        result.school_name = first_record.unit_shortname

    return result


def _get_data(date_from, date_to, event_type):
    u"""Возвращает данные о количестве аутентификаций сотрудников.

    :param date_from: Дата начала периода, за который нужно загрузить
        информацию.
    :type date_from: datetime.date

    :param date_to: Дата окончания периода, за который нужно загрузить
        информацию.
    :type date_to: datetime.date

    :param str event_type: Тип события в журнале событий (``AUTH_LOGIN`` или
        ``AUTH_ESIA_LOGIN``).

    :rtype: :class:`edu_to_analytics.service.types.Root`
    """
    datetime_range = (
        datetime.combine(date_from, time()),
        datetime.combine(date_to, time(23, 59, 59, 999999)),
    )

    employee_users = Employee.objects.values_list(
        'person__user', flat=True
    ).distinct()

    log_records = EventLog.objects.filter(
        user_id__in=tuple(employee_users),
        module=MODULE_AUTH,
        event=event_type,
        datetime__range=datetime_range,
    ).order_by('user_id', 'datetime')

    employees = []
    for (user_id, day), records in groupby(
        log_records, lambda r: (r.user_id, r.datetime.date())
    ):
        employees.append(_build_record(user_id, day, tuple(records)))

    result = Root()
    result.employees = employees
    return result


def get_authentications_info(date_from, date_to, **kwargs):
    u"""Возвращает данные о количестве аутентификаций через форму входа в ЭДО.

    :param date_from: Дата начала периода, за который нужно загрузить
        информацию.
    :type date_from: datetime.date

    :param date_to: Дата окончания периода, за который нужно загрузить
        информацию.
    :type date_to: datetime.date

    :rtype: :class:`edu_to_analytics.service.types.Root`
    """
    return _get_data(date_from, date_to, AUTH_LOGIN)


def get_esia_authentications_info(date_from, date_to, **kwargs):
    u"""Возвращает данные о количестве аутентификаций через ЕСИА.

    :param date_from: Дата начала периода, за который нужно загрузить
        информацию.
    :type date_from: datetime.date

    :param date_to: Дата окончания периода, за который нужно загрузить
        информацию.
    :type date_to: datetime.date

    :rtype: :class:`edu_to_analytics.service.types.Root`
    """
    return _get_data(date_from, date_to, AUTH_LOGIN_ESIA)


CODE_METHOD_MAP = {
    'LoginCounter': get_authentications_info,
    'ESIALoginCounter': get_esia_authentications_info
}
def get_content(out_format=None, **kwargs):
    u"""Получает и сериализует данные в указанный формат."""

    out_format = out_format or 'xml'

    root = CODE_METHOD_MAP[kwargs['report_code']](**kwargs)

    if out_format == 'xml':
        result = tostring(
            get_object_as_xml(root), pretty_print=True, encoding='unicode'
        )
    else:
        raise NotImplementedError(
            '{} format is not implemented'.format(out_format)
        )

    return result


def validate_request(ctx, report_code, period):
    u"""Проверяет входные данные."""
    error = None
    period_type = period.IndexPeriodType

    if period_type == PERIOD_DATE and not period.IndexPeriodDate:
        error = (
            u'Для периода `date` должен быть указан параметр `IndexPeriodDate`'
        )

    elif (period_type == PERIOD_PERIOD and not
          all((period.IndexPeriodStartDate, period.IndexPeriodEndDate))):
        error = (
            u'Для периода `period` должны быть указаны параметры '
            u'`IndexPeriodStartDate` и `IndexPeriodEndDate`'
        )

    if error is not None:
        raise SpyneException(code=error)
